package cn.shuzilm.odps;

import cn.shuzilm.odps.DnaUtils;
import cn.shuzilm.odps.Utils;
import com.aliyun.odps.OdpsException;
import com.aliyun.odps.data.Record;
import com.aliyun.odps.data.TableInfo;
import com.aliyun.odps.mapred.JobClient;
import com.aliyun.odps.mapred.MapperBase;
import com.aliyun.odps.mapred.Mapper.TaskContext;
import com.aliyun.odps.mapred.conf.JobConf;
import com.aliyun.odps.mapred.utils.InputUtils;
import com.aliyun.odps.mapred.utils.OutputUtils;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

public class OdsDnaDetail2D {

   public static void main(String[] args) throws OdpsException {
      if(args.length < 10) {
         System.err.println("Usage:cn.shuzilm.odps.Ods_dna_export stg_dna_info_h dna_extract 20150101 20160101 pkg_name channels is_all_data has_update");
         System.err.println(args.length);
         System.exit(1);
      }

      String odsDnaInfoTable = args[0];
      String outPutTbl = args[1];
      String startTime = args[2];
      String endTime = args[3];
      String pkgNames = args[4].toLowerCase();
      String channels = args[5];
      String ver_names = args[6];
      String columns = args[7];
      String rule_no_invalid = args[8];
      String show_state_list = args[9];
      String if_show_whole = args[10];
      String CDID = args[11];
      String DID = args[12];
      String IP = args[13];
      String cid_in = args[14].toLowerCase();
      String start_day = startTime.substring(0, 8);
      String start_date_time_str = parseDate(startTime, "yyyyMMddHH", "yyyy-MM-dd HH:mm:ss");
      String end_day = endTime.substring(0, 8);
      String end_date_time_str = parseDate(endTime, "yyyyMMddHH", "yyyy-MM-dd HH:mm:ss");
      JobConf job = new JobConf();
      job.set("ods_dna_info_table", odsDnaInfoTable);
      job.set("start_time", startTime);
      job.set("end_time", endTime);
      job.set("pkg", pkgNames);
      job.set("filter_cid", cid_in);
      job.set("channel_list", channels);
      job.set("ver_name_list", ver_names);
      job.set("column_list", columns);
      if(!rule_no_invalid.equals("rule_no_empty")) {
         job.set("rule_no_invalid_list", rule_no_invalid);
      }

      job.set("show_state_list", show_state_list);
      job.set("if_show_whole", if_show_whole);
      if(!CDID.equals("cdid_empty")) {
         job.set("cdid_list", CDID);
      }

      if(!DID.equals("did_empty")) {
         job.set("did_list", DID);
      }

      if(!IP.equals("ip_empty")) {
         job.set("ip_list", IP);
      }

      job.setMapperClass(OdsDnaDetail2D.OdsDnaDetail2dMapper.class);
      job.setNumReduceTasks(0);
      DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyyMMdd");
      DateTime dateTime = fmt.parseDateTime(start_day);

      for(DateTime end_dateTime = fmt.parseDateTime(end_day); dateTime.compareTo(end_dateTime) <= 0; dateTime = dateTime.plusDays(1)) {
         InputUtils.addTable(TableInfo.builder().tableName(odsDnaInfoTable).partSpec("day=" + dateTime.toString(fmt)).label("in").build(), job);
      }

      InputUtils.addTable(TableInfo.builder().tableName(outPutTbl).label("out").build(), job);
      OutputUtils.addTable(TableInfo.builder().tableName(outPutTbl).build(), job);
      JobClient.runJob(job);
   }

   private static String parseDate(String date, String old_format, String new_format) {
      try {
         if(StringUtils.isNotBlank(date)) {
            SimpleDateFormat e = new SimpleDateFormat(old_format);
            SimpleDateFormat newFormat = new SimpleDateFormat(new_format);
            Date d = e.parse(date);
            String dateString = newFormat.format(d);
            return dateString;
         }
      } catch (ParseException var7) {
         var7.printStackTrace();
      }

      return "";
   }

   public static class OdsDnaDetail2dMapper extends MapperBase {

      private Record result;
      private String odsDnaInfoTable;
      private String filter_cid;
      private ArrayList pkgList = new ArrayList();
      private ArrayList channel_list = new ArrayList();
      private ArrayList ver_name_list = new ArrayList();
      private ArrayList ip_list = new ArrayList();
      private ArrayList did_list = new ArrayList();
      private ArrayList cdid_list = new ArrayList();
      private ArrayList rule_no_invalid_list = new ArrayList();
      private ArrayList show_state_list = new ArrayList();
      private ArrayList column_list = new ArrayList();
      private static int SHOW_ALL_STATE = 0;
      public static int NOT_SHOW_ALL_STATE = 1;
      private String start_time = "";
      private String end_time = "";
      private Boolean tag;


      public void setup(TaskContext context) throws IOException {
         this.tag = Boolean.valueOf(context.getInputTableInfo().getLabel().equals("out"));
         this.result = context.createOutputRecord();
         String pkgListStr = context.getJobConf().get("pkg");
         this.filter_cid = context.getJobConf().get("filter_cid");
         this.odsDnaInfoTable = context.getJobConf().get("ods_dna_info_table");
         String channels = context.getJobConf().get("channel_list");
         String if_show_whole = context.getJobConf().get("if_show_whole");
         if(!StringUtils.isBlank(pkgListStr) && pkgListStr.contains(".")) {
            this.pkgList.addAll(Arrays.asList(pkgListStr.split(",")));
         }

         int show_state_str;
         String ip_str;
         if(channels.equals("chl_names")) {
            this.channel_list = null;
         } else {
            String[] ver_names = channels.split(",");
            int columns_string = ver_names.length;

            for(show_state_str = 0; show_state_str < columns_string; ++show_state_str) {
               ip_str = ver_names[show_state_str];
               if(ip_str != null && !ip_str.isEmpty() && !"null".equals(ip_str.toLowerCase())) {
                  this.channel_list.add(ip_str.toLowerCase());
               }
            }
         }

         String var12 = context.getJobConf().get("ver_name_list");
         String did_str;
         if(var12.equals("ver_names")) {
            this.ver_name_list = null;
         } else {
            String[] var13 = var12.split(",");
            show_state_str = var13.length;

            for(int var16 = 0; var16 < show_state_str; ++var16) {
               did_str = var13[var16];
               if(did_str != null && !did_str.isEmpty() && !"null".equals(did_str.toLowerCase())) {
                  this.ver_name_list.add(did_str.toLowerCase());
               }
            }
         }

         String var14 = context.getJobConf().get("column_list").toLowerCase();
         Collections.addAll(this.column_list, var14.split(","));
         String var15 = context.getJobConf().get("show_state_list");
         String[] var17 = var15.split(",");
         int var18 = var17.length;

         String rule_no_invalid_str;
         for(int cdid_str = 0; cdid_str < var18; ++cdid_str) {
            rule_no_invalid_str = var17[cdid_str];
            this.show_state_list.add(DnaUtils.parseAndroidStates(rule_no_invalid_str));
         }

         if(Integer.parseInt(if_show_whole) == SHOW_ALL_STATE) {
            this.show_state_list.add("模块异常");
            this.show_state_list.add("无应用版本号");
            this.show_state_list.add("二代ID不合规");
            this.show_state_list.add("二代DID在库中没找到");
            this.show_state_list.add("重复上报");
         }

         ip_str = context.getJobConf().get("ip_list");
         if(ip_str != null && !ip_str.equals("ip_empty")) {
            Collections.addAll(this.ip_list, ip_str.split(","));
         }

         did_str = context.getJobConf().get("did_list");
         if(did_str != null && !did_str.isEmpty() && !"null".equals(did_str.toLowerCase()) && !"did_empty".equals(did_str)) {
            Collections.addAll(this.did_list, did_str.split(","));
         }

         String var19 = context.getJobConf().get("cdid_list");
         if(var19 != null && !var19.isEmpty() && !"null".equals(var19.toLowerCase()) && !"cdid_empty".equals(var19)) {
            Collections.addAll(this.cdid_list, var19.split(","));
         }

         rule_no_invalid_str = context.getJobConf().get("rule_no_invalid_list");
         if(rule_no_invalid_str != null && !rule_no_invalid_str.equals("rule_no_empty")) {
            Collections.addAll(this.rule_no_invalid_list, rule_no_invalid_str.split(","));
         }

         this.start_time = context.getJobConf().get("start_time");
         this.end_time = context.getJobConf().get("end_time");
         System.err.println(pkgListStr + "\n" + channels + "\n" + var12 + "\n");
      }

      public void map(long recordNum, Record record, TaskContext context) throws IOException {
         try {
            if(this.tag.booleanValue()) {
               context.write(record);
            } else if(this.if_filter_pass(record)) {
               this.fill_data(record, this.column_list);
               context.write(this.result);
            }
         } catch (Exception var6) {
            var6.printStackTrace();
         }

      }

      public static boolean time_fit(String ts, String start_time, String end_time) {
         if(StringUtils.isNotBlank(start_time) && StringUtils.isNotBlank(end_time) && StringUtils.isNotBlank(ts)) {
            String databaseTs = OdsDnaDetail2D.parseDate(ts, "yyyy-MM-dd HH:mm:ss", "yyyyMMddHH");
            return databaseTs.compareTo(start_time) >= 0 && databaseTs.compareTo(end_time) < 0;
         } else {
            return false;
         }
      }

      private boolean if_filter_pass(Record record) {
         String pkg_name = record.getString("pkg_name");
         String cid = record.getString("cid");
         String store = record.getString("store");
         if(store != null) {
            store = store.toLowerCase();
         }

         String ver_name = record.getString("ver__name");
         String ts = record.getString("time");
         String ip = record.getString("ip");
         String device_type = record.getString("device_type");
         String app_type = record.getString("app_type");
         String times_result = DnaUtils.parseAndroidTimes(device_type, app_type);
         if(this.show_state_list != null && this.show_state_list.size() > 0 && !this.show_state_list.contains(times_result)) {
            System.out.println(String.format("8 ---device_type: %s, app_type: %s ,times_result: %s", new Object[]{device_type, app_type, times_result}));
            return false;
         } else if(pkg_name != null && !this.pkgList.isEmpty() && !this.pkgList.contains(pkg_name.toLowerCase())) {
            System.out.println(String.format("1-------pkg_name: %s", new Object[]{pkg_name}));
            return false;
         } else if(cid != null && !this.filter_cid.isEmpty() && !this.filter_cid.equals(cid.toLowerCase())) {
            System.out.println(String.format("x-------cid: %s", new Object[]{cid}));
            return false;
         } else if(this.channel_list != null && this.channel_list.size() > 0 && !this.channel_list.contains(store)) {
            System.out.println("2");
            return false;
         } else if(this.ver_name_list != null && this.ver_name_list.size() > 0 && !this.ver_name_list.contains(ver_name)) {
            System.out.println("3");
            return false;
         } else if(!time_fit(ts, this.start_time, this.end_time)) {
            System.out.println("4");
            return false;
         } else {
            String rule_no_invalid;
            String cdid2;
            if("ods_dna_info_d_new".equals(this.odsDnaInfoTable)) {
               rule_no_invalid = record.getString("did");
               if(this.did_list != null && this.did_list.size() > 0 && !this.did_list.contains(rule_no_invalid)) {
                  System.out.println("5");
                  return false;
               }
            } else if("ods_dna_info_h_2d".equals(this.odsDnaInfoTable)) {
               rule_no_invalid = record.getString("issue_id1");
               cdid2 = record.getString("issue_id2");
               if(this.did_list != null && this.did_list.size() > 0 && !this.did_list.contains(rule_no_invalid) && !this.did_list.contains(cdid2)) {
                  System.out.println("5");
                  return false;
               }
            }

            if("ods_dna_info_d_new".equals(this.odsDnaInfoTable)) {
               rule_no_invalid = record.getString("ldid");
               if(this.cdid_list != null && this.cdid_list.size() > 0 && !this.cdid_list.contains(rule_no_invalid)) {
                  System.out.println("6");
                  return false;
               }
            } else if("ods_dna_info_h_2d".equals(this.odsDnaInfoTable)) {
               rule_no_invalid = record.getString("cdid1");
               cdid2 = record.getString("cdid");
               if(this.cdid_list != null && this.cdid_list.size() > 0 && !this.cdid_list.contains(rule_no_invalid) && !this.cdid_list.contains(cdid2)) {
                  System.out.println("6");
                  return false;
               }
            }

            if(this.ip_list != null && this.ip_list.size() > 0 && !this.ip_list.contains(ip)) {
               System.out.println("7");
               return false;
            } else {
               rule_no_invalid = "";
               if("ods_dna_info_d_new".equals(this.odsDnaInfoTable)) {
                  rule_no_invalid = record.getString("rule_no_invalid");
               } else if("ods_dna_info_h_2d".equals(this.odsDnaInfoTable)) {
                  rule_no_invalid = record.getString("rule_no");
               }

               if(this.rule_no_invalid_list != null && this.rule_no_invalid_list.size() > 0) {
                  Iterator cdid21 = this.rule_no_invalid_list.iterator();

                  while(cdid21.hasNext()) {
                     String flow_rule_no_invalid = (String)cdid21.next();
                     if(rule_no_invalid.contains(flow_rule_no_invalid)) {
                        return false;
                     }
                  }
               }

               System.out.println("************************");
               return true;
            }
         }
      }

      private void fill_data(Record record, ArrayList column_list) {
         int j = 0;
         String cid = record.getString("cid");
         String sdk_ver = record.getString("sdk__ver");
         Iterator var6 = column_list.iterator();

         while(var6.hasNext()) {
            String column = (String)var6.next();

            try {
               String e;
               if(column.equals("status")) {
                  e = record.getString("device_type");
                  String app_type = record.getString("app_type");
                  String times_result = DnaUtils.parseAndroidTimes(e, app_type);
                  this.result.set(j++, times_result);
               } else if(column.equals("device_verify")) {
                  this.result.set(j++, (Object)null);
               } else if(!column.equals("curtime") && !column.equals("first__install__time") && !column.equals("last__update__time")) {
                  if(column.equals("mac__addr") && "MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBANc7lrAPh8Vki2+Gf9KQxUbWzTqMvwIe+y9VuHfUPWHlTovg3JwUJnkoTHLCTH2BTMHHRe/lkEFCn7wAXmrisjsCAwEAAQ==".equals(cid) && sdk_ver.compareTo("v6.1.6") >= 0) {
                     this.result.set(j++, (Object)null);
                  } else if(column.equals("tmp_did")) {
                     this.result.set(j++, record.getString("did"));
                  } else {
                     this.result.set(j++, record.getString(column));
                  }
               } else {
                  e = record.getString(column);
                  this.result.set(j++, Utils.timestamp_2_date(e));
               }
            } catch (NullPointerException var11) {
               ++j;
               System.out.println("null , " + j + " , " + column);
            } catch (Exception var12) {
               var12.printStackTrace();
               System.out.println("null , " + j + " , " + column);
            }
         }

      }

   }
}
